package eu.jonahbauer.raytracing.render.renderer.pdf;

import eu.jonahbauer.raytracing.math.Vec3;
import org.jetbrains.annotations.NotNull;

import java.util.random.RandomGenerator;

/**
 * A probability density function used for sampling random directions when scattering a ray.
 */
public interface ProbabilityDensityFunction {

    /**
     * {@return the value of this probability density function at the given point}
     * @param direction the direction
     */
    double value(@NotNull Vec3 direction);

    /**
     * Generates a random direction based on this probability density function.
     * @param random a random number generator
     * @return the random direction
     */
    @NotNull Vec3 generate(@NotNull RandomGenerator random);
}
