package eu.jonahbauer.raytracing.scene.util;

import eu.jonahbauer.raytracing.math.Vec3;
import org.jetbrains.annotations.NotNull;

public final class PdfUtil {
    private PdfUtil() {
        throw new UnsupportedOperationException();
    }

    /**
     * {@return the solid angle of the triangle abc as seen from the origin} The vectors {@code a}, {@code b} and {@code c}
     * must be unit vectors.
     */
    public static double getSolidAngle(@NotNull Vec3 a, @NotNull Vec3 b, @NotNull Vec3 c) {
        var angle = 2 * Math.atan(Math.abs(a.times(b.cross(c))) / (1 + a.times(b) + b.times(c) + c.times(a)));
        return angle < 0 ? 2 * Math.PI + angle : angle;
    }
}
